<?php

namespace Drupal\cura\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * General settings for the Cura Childcare Suite.
 */
class GeneralSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'cura_general_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['cura.general.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('cura.general.settings');

    $form['settings'] = [
      '#type' => 'vertical_tabs',
      '#default_tab' => 'org',
    ];

    // Organisation settings.
    $form['org'] = [
      '#type' => 'details',
      '#title' => $this->t('Organisation'),
      '#tree' => TRUE,
      '#group' => 'settings',
    ];
    $form['org']['name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Organisation name'),
      '#default_value' => $config->get('org.name'),
    ];
    $form['org']['name_short'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Short name'),
      '#default_value' => $config->get('org.name_short'),
    ];
    $types = ['Charity', 'Company'];
    $form['org']['type'] = [
      '#type' => 'select',
      '#title' => $this->t('Type of organisation'),
      '#options' => array_combine($types, $types),
      '#default_value' => $config->get('org.type'),
    ];
    $form['org']['number'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Organisation number'),
      '#default_value' => $config->get('org.number'),
      '#size' => 20,
      '#maxlength' => 30,
    ];
    $form['org']['number_ofsted'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Ofsted URN'),
      '#default_value' => $config->get('org.number_ofsted'),
      '#size' => 10,
      '#maxlength' => 10,
    ];

    // Bank settings.
    $form['bank'] = [
      '#type' => 'details',
      '#title' => $this->t('Bank'),
      '#tree' => TRUE,
      '#group' => 'settings',
    ];
    $form['bank']['account_bank'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Bank/Institution'),
      '#default_value' => $config->get('bank.account_bank'),
    ];
    $form['bank']['account_branch'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Branch office'),
      '#default_value' => $config->get('bank.account_branch'),
    ];
    $form['bank']['account_owner'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Account owner'),
      '#description' => $this->t('Name associated with the bank account.'),
      '#default_value' => $config->get('bank.account_owner'),
    ];
    $form['bank']['account_number'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Account number'),
      '#default_value' => $config->get('bank.account_number'),
    ];
    $form['bank']['account_iban'] = [
      '#type' => 'textfield',
      '#title' => $this->t('IBAN'),
      '#description' => $this->t('International Bank Account Number'),
      '#default_value' => $config->get('bank.account_iban'),
    ];
    $form['bank']['code_appellation'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Bank code appellation'),
      '#description' => $this->t('Appellation of bank code - depending on where your bank is located you should set this to: "BSB" (AU), "Sort code" (UK), "Bank code" (DE), "Clearing number" (CH), "Routing transit number" (US) or "Bank transit number" (CA)'),
      '#default_value' => $config->get('bank.code_appellation'),
    ];
    $form['bank']['code'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Bank code'),
      '#description' => $this->t('Actual bank code, will be shown with appellation set above. In the UK this will be the sort code.'),
      '#default_value' => $config->get('bank.code'),
    ];
    $form['bank']['account_swift'] = [
      '#type' => 'textfield',
      '#title' => $this->t('SWIFT'),
      '#description' => $this->t('SWIFT-Code (aka BIC = Bank Identifier Code)'),
      '#default_value' => $config->get('bank.account_swift'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Retrieve the configuration
    // drupal_set_message(print_r($form_state->getValues(), TRUE));
    $this->configFactory->getEditable('cura.general.settings')
      ->set('org.name', $form_state->getValue(array('org', 'name')))
      ->set('org.name_short', $form_state->getValue(array('org', 'name_short')))
      ->set('org.type', $form_state->getValue(array('org', 'type')))
      ->set('org.number', $form_state->getValue(array('org', 'number')))
      ->set('org.number_ofsted', $form_state->getValue(array('org', 'number_ofsted')))
      ->set('bank', $form_state->getValue(array('bank')))
      ->save();

    parent::submitForm($form, $form_state);
  }
}

